/*

	QuickPasswords
	created by Axel Grude

	Version History
	0.9 - 10/02/2010
		First Release on AMO
	0.9.3 - 10/02/2010
		Fixed localization issues
		Fixed an issue with case sensitivity in Linux
		Added missing icon for small toolbar view
		Added 'real' settings to options dialog
	0.9.4 - 12/02/2010
		Fixed Icon and Writing on Browser Content Popup Menu
		Added links to new extension homepage:	 http://quickpasswords.mozdev.org
		Removed any references to the old working title "PasswordClipper"
	0.9.5 - 14/02/2010
		Added German Locale (AG)
		Added French Locale (Bilouba http://lionel.bijaoui.free.fr/)
		Added feature context menu in password manager
		Added copy row feature
		Added about button to options dialog
	0.9.6 - 20/02/2010
		Added "copy multiple lines / records" feature
		Added: option to include header (for pasting data in spreadsheets) when doing this
		Added: Log copied information about copied data to javascript console (passwords are not displayed)
		Added Dutch Locale (markh van BabelZilla.org)
		Added: Localization in extension description strings
	0.9.6.1 - 27/02/2010
		markh: Reviewed Dutch locale
		Added link to Mozdev homepage to options dialog
		removed JSON module
		Added Russian Locale (Anton Pinsky)
		Added Simplified Chinese Locale (Loviny)
		Added About button + Full Version number to Options screen
	0.9.7 - 07/03/2010
		Added Option for automatic closing of Password Manager after copying
		Added Vietnamese locale (Nguyen Hoang Long http://timelinelive.blogspot.com)
		Added Hungarian locale (WonderCsabo of Babelzilla.org)
	0.9.8 - 01/06/2010
		New Options for copying User and Site
	1.0 - 08/07/2010
		Added da locale (Jorgen Rasmussen)
	1.1 - 29/12/2010
		Fixed a bug that made Header row appear as "undefined" when copying multiple records and "Display message after copying" was disabled
		Made context menu compatible with the "Saved Password Editor" Extension
	1.2 - 04/01/2011
		Made Extension compatible with Firefox 4.0
		Added he-IL Locale (barryoni)
	1.3 - 22/01/2011
		Removed a bug "menu is null" when opening password manager window
		Added Options Menu Item
		Added option to hide Context Menu
		Context Menu item in browser window is now only visible on editable elements
		Added es-AR locale (Eduardo Leon)
		Added ja-JP locale (Noumi Ryoko)
		Added sv-SE locale (Mikael Hiort af Ornaes)
	1.4 - 10/02/2011
		Force icons in context menu even in default theme (class menuitem-iconic)
		Added tooltip (and updated label text) to the time delay option
		Now there are 3 options for displaying browser context menu: never, on text items and always
		open issues: in Fx4, I am still not able to display the extension's version number!
	1.5 - WIP
		Added Fx 5.0* compatibility

===========================================================================================

		Planned: adding option for configuring delimiter of multi line export. At the moment this is hard coded to tab.

*/


var CC_loginManager = Components.classes["@mozilla.org/login-manager;1"]; // Fx 3
var CC_passwordManager = Components.classes["@mozilla.org/passwordmanager;1"]; // Fx 2 (TB?)

var QuickPasswords_BundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
var QuickPasswords_Bundle = QuickPasswords_BundleService.createBundle("chrome://quickpasswords/locale/overlay.properties");



var QuickPasswords = {
	strings:null,
	initialized:false,
	PasswordManagerWindow: null,
	onLoad: function() {
		if(this.initialized)
			return;
		//http://kb.mozillazine.org/Adding_items_to_menus
		var menu = document.getElementById("contentAreaContextMenu");
		// no need to do this when coming back from password window, document context is wrong then and throws an error
		if (menu) {
			menu.addEventListener("popupshowing", QuickPasswords.contextPopupShowing, false);
		}

		QuickPasswords_BundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
		QuickPasswords_Bundle = QuickPasswords_BundleService.createBundle("chrome://quickpasswords/locale/overlay.properties");
		this.strings = document.getElementById("QuickPasswords-strings");
		QuickPasswords.Util.logDebug("QuickPasswords " + QuickPasswords.Util.ExtensionVersion()
			 + " running on " + QuickPasswords.Util.Application()
			 + " Version " + QuickPasswords.Util.AppverFull() + "."
			 + "\nOS: " + QuickPasswords.Util.HostSystem());
		this.initialized = true;
	},

	contextPopupShowing: function() {
		// only show if a text item is right-clicked and also the option must be checked.
		// 0 = always
		// 1 = selective
		// 2 = never
		var show = !(QuickPasswords.Preferences.contextMenuOption() == 2) &&
				(
					(QuickPasswords.Preferences.contextMenuOption() == 0) ||
					((QuickPasswords.Preferences.contextMenuOption() == 1) && gContextMenu.onTextInput)
				)


		gContextMenu.showItem("context-quickPasswords", show);
	},

	getOpenWindow: function(name) {
		var mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
		var win = mediator.getMostRecentWindow(name);
	},

	showPasswords: function(filterString) {
	const name = "Toolkit:PasswordManager";
	const uri = "chrome://passwordmgr/content/passwordManager.xul";

	var win = this.getOpenWindow(name);

	if (!win) {
		var argstring;
		argstring = Components.classes["@mozilla.org/supports-string;1"]
						.createInstance(Components.interfaces.nsISupportsString);
		argstring.data = filterString;


		var watcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].getService(Components.interfaces.nsIWindowWatcher);
		win = watcher.openWindow(null, uri, name, "chrome,centerscreen,resizable,alwaysRaised", argstring); //
	}

	this.PasswordManagerWindow = win;

	win.setTimeout(
		function () {
			if (win.self.setFilter)
				win.self.setFilter(filterString);
		}, QuickPasswords.Preferences.waitForManager());


	},

	getManagerColumn:   function (idx, colName) { return self.signonsTreeView.getCellText(idx,self.signonsTree.columns.getNamedColumn(colName)); },
	getSite:     function (idx) { return this.getManagerColumn (idx, 'siteCol');     },
	getUser:     function (idx) { return this.getManagerColumn (idx, 'userCol');     },
	getPassword: function (idx) { return this.getManagerColumn (idx, 'passwordCol'); },


	copyToClipboard : function(what) {

		const name = "Toolkit:PasswordManager";
		var lstPasswords = document.getElementById('signonsTree'); // password list
		var treeView = self.signonsTreeView;
		var tree = self.signonsTree;

		if (tree.currentIndex<0) {
			var msg = QuickPasswords_Bundle.GetStringFromName("selectOneMessage");
			alert (msg);
		}


		var site = this.getSite(tree.currentIndex);
		var user = this.getUser(tree.currentIndex);
		var pwd = this.getPassword(tree.currentIndex);
		var cm,cu,cs,cp,ct;
		var sCopyDummy="";

		if (QuickPasswords.Preferences.isCopyMsg() || what=='row') {
			try {
				// in SeaMonkey, this fails for some reason
				cm = QuickPasswords_Bundle.GetStringFromName("copyMessage");
				cu = QuickPasswords_Bundle.GetStringFromName("copyMessageUser");
				cs = QuickPasswords_Bundle.GetStringFromName("copyMessageDomain");
				cp = QuickPasswords_Bundle.GetStringFromName("copyMessagePassword");
				ct = QuickPasswords_Bundle.GetStringFromName("copyMessageTitle"); // currently unused
			}
			catch (ex)
			{
				QuickPasswords.Util.logToConsole("Error trying to get string bundle: " + ex);
				cm = "Copied Password to Clipboard!";
				cu = "User";
				cs = "Site";
				ct = "QuickPasswords";
				cp = "Password";
			}
		}

		switch (what) {
			case 'pwd':
				QuickPasswords.Util.copyStringToClipboard(pwd);
				if (QuickPasswords.Preferences.isCopyMsg())
					alert (cm + '\n' + cu + ': ' + user + '\n' + cs + ': ' + site);
				break;

			case 'url':
				QuickPasswords.Util.copyStringToClipboard(site);

				if (QuickPasswords.Preferences.isCopyMsg())
					alert (cm.replace(new RegExp(cp, "i" ), cs) + '\n' + cu + ': ' + user + '\n' + cs + ': ' + site); // replace case insensitive!
				break;


			case 'usr':
				QuickPasswords.Util.copyStringToClipboard(user);

				if (QuickPasswords.Preferences.isCopyMsg())
					alert (cm.replace(new RegExp(cp, "i" ), cu) + '\n' + cu + ': ' + user + '\n' + cs + ': ' + site); // replace case insensitive!
				break;

			case 'row':
				var iPasswordsSelected = 0;
				var start = new Object();
				var end = new Object();
				var numRanges = tree.view.selection.getRangeCount();
				var sLine = '';
				var sCopyText='';

				for (var t = 0; t < numRanges; t++){
					tree.view.selection.getRangeAt(t,start,end);
					for (var v = start.value; v <= end.value; v++){
						iPasswordsSelected++;
						site = this.getSite(v);
						user = this.getUser(v);
						pwd = this.getPassword(v);
						sLine = site + '\t' + user + '\t' + pwd;
						sCopyText = sCopyText + sLine + '\n';
						sCopyDummy = sCopyDummy + site + '\t' + user + '\t' + '*********\n';
					}
				}
				if (iPasswordsSelected==0) {
					cm = QuickPasswords_Bundle.GetStringFromName("selectOneMessage");
					alert (cm);
					return;
				}

				if (iPasswordsSelected > 1) {
					if (QuickPasswords.Preferences.isMultiRowHeader())
						sCopyText = cs + '\t' + cu + '\t' + cp + '\n' + sCopyText;
					sCopyDummy = cs + '\t' + cu + '\t' + cp + '\n' + sCopyDummy;

					QuickPasswords.Util.copyStringToClipboard(sCopyText);
				}
				else
					QuickPasswords.Util.copyStringToClipboard(site + '\t' + user + '\t' + pwd);
				if (QuickPasswords.Preferences.isCopyMsg()) {
					try {
						if (1==iPasswordsSelected)
							cm = QuickPasswords_Bundle.GetStringFromName("copyRecordMessage");
						else
							cm = QuickPasswords_Bundle.GetStringFromName("copyRecordsMessage");
					}
					catch(e) {
						QuickPasswords.Util.logToConsole("Error getting String Bundle GetStringFromName\n" + e);
						cm="Data copied to clipboard - see error console for detail";
					}
					alert (cm);
				}
				break;

		}
		if (""!=sCopyDummy)
			QuickPasswords.Util.logToConsole("Copied Password Information to Clipboard:\n" + sCopyDummy);
		else
			QuickPasswords.Util.logToConsole("Copied Password Information to Clipboard for user: " + user + " at site: " + site);

		if (QuickPasswords.Preferences.isAutoClose()) {
			if	(null==this.PasswordManagerWindow)
				this.PasswordManagerWindow=self;
			this.PasswordManagerWindow.close();
		}
		this.PasswordManagerWindow=null;
		},

		getURIcontext: function() {
		var sBaseDomain;

	/*	try {
			var tldService =
				Components.classes['@mozilla.org/network/effective-tld-service;1']
					.getService(Components.interfaces.nsIEffectiveTLDService);

			sBaseDomain = tldService.getBaseDomain(content.document.documentURIObject);
		}
		catch(ex) {
			QuickPasswords.Util.logDebug("Error in getURIcontext\n" + ex); */
			var sHost=content.location.host;
			var pos = sHost.indexOf('.');
			var s2 = sHost.substring(pos+1, sHost.length);
			var pos2 = s2.indexOf('.');
			// strip first word if second is long enough to be a domain; gets rid of stuff like www. login. etc.
			if (pos2>3)
				sHost=sHost.substring(pos+1,sHost.length);
			sBaseDomain = sHost;
			QuickPasswords.Util.logDebug("Determined Domain from Host String:" + sBaseDomain);
	//	}

		return sBaseDomain;
	},

	attemptLogin: function() {
		var site = this.getSite(tree.currentIndex);
		var user = this.getUser(tree.currentIndex);
		var pwd = this.getPassword(tree.currentIndex);

		if (tree.currentIndex<0) {
			var msg = QuickPasswords_Bundle.GetStringFromName("selectOneMessage");
			alert (msg);
		}


	},

	onMenuItemCommand: function(e) {
		var filter;

		switch (QuickPasswords.Util.Application()) {
			case 'Thunderbird': // try	to use current mail account name as filter?
				filter = '';
				break;
			case 'SeaMonkey':
				// we need to find out whether I am called from the messenger window or from navigator!
				try { filter = this.getURIcontext(); }
				catch(er) { filter = ''; }
				break;
			case 'Firefox': // use current address bar address, if in
				filter = this.getURIcontext();
				break;
		}
		this.showPasswords(filter);

		// if (this.strings==null) this.onLoad();
		/*promptService.alert(window, this.strings.getString("helloMessageTitle"),
									this.strings.getString("helloMessage")); */
	},

	onContextMenuCommand: function(e) {
		promptService.alert(window,"test","onContextMenuCommand...");
		this.showPasswords('test.com'); // filter string from id of right click item! or from current domain?
	},

	onToolbarButtonCommand: function(e) {
		// just reuse the function above.	you can change this, obviously!
		QuickPasswords.onMenuItemCommand(e);
	}
};

window.addEventListener("load", QuickPasswords.onLoad, false);
